//    Austin is a program for updating projects kept in XML files.
//    Copyright (C) 2000  Joseph Gregorio
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef AUSTIN_JULIANDAY_H
#define AUSITN_JULIANDAY_H

// put class declarations here
#include <string>
using namespace std;

#pragma warning(disable:4231)
#pragma warning(disable:4251)

// The default Julian Day Number used when no date or an invalid date is specified.
// It is based on a date of 1/1/1901.
const double DEFUALT_JULIAN_DAY_NUMBER = 2415385.5;

class JulianDay;

bool operator== (const JulianDay & lhs, const JulianDay & rhs);
bool operator< (const JulianDay & lhs, const JulianDay & rhs);
bool operator> (const JulianDay & lhs, const JulianDay & rhs);
bool operator<= (const JulianDay & lhs, const JulianDay & rhs);
bool operator>= (const JulianDay & lhs, const JulianDay & rhs);
bool operator!= (const JulianDay & lhs, const JulianDay & rhs);
int operator- (const JulianDay & lhs, const JulianDay & rhs);


// This class Defines a Julian Day Number for a date. This value 
//	can be used for easily comparing dates. All dates are assumed to be on the Gregorian
//	Calendar. Results will not be accurate on years earlier than 1582 AD.
//
// 
//  If an invalid date is passed into a constructor or setDate function, the program
//	will assert if in debug mode or if not in debug mode the default value will be 
//	assigned.
//
class  JulianDay {
	protected:

		// Stores the value of the Julian Day Number
		double julianDayNumber_;

	public:
		JulianDay();

		JulianDay(const struct tm *);

		JulianDay(const double julianDayNumber);

		JulianDay(const JulianDay& julianDay);

		~JulianDay();

		// Retrieves the string representation of the date
		//	represented by the current Julian Day Number. 
		wstring asString() const;

		// Retrieves the value of this Julian Day Number.
		double asDouble() const;

		// Retrieves the value of this Julian Day Number.
		struct tm asTm() const;

		// Updater.
		void set(const wstring &);

		// Sets the Juilan Day Number to the given date.
		void set(const struct tm *);

		// Sets this Juilan Day Number to the given value.
		void set(double julianDayNumber);

		// Assignment operator.
		JulianDay & operator= (const JulianDay & julianDay);

		// Add the given number of days to this JulianDay and return the
		//	resulting JulianDay.
		JulianDay operator+ (int days);

		// Add the given number of days to this JulianDay.
		void operator+= (int days);

		// Increment this JulianDay by 1 day. Prefix
		JulianDay & operator++ ();

		// Increment this JulianDay by 1 day. Postfix
		JulianDay operator++ (int);

		// Subtract the given number of days to this JulianDay and return the 
		//	resulting JulianDay.
		JulianDay operator- (int days);

		// Subtract the given number of days to this JulianDay.
		void operator-= (int days);

		// Decrement this JulianDay by 1 day. Prefix
		JulianDay & operator-- ();

		// Decrement this JulianDay by 1 day. Postfix
		JulianDay operator-- (int);

		friend bool operator== (const JulianDay & lhs, const JulianDay & rhs);
		friend bool operator< (const JulianDay & lhs, const JulianDay & rhs);
		friend bool operator> (const JulianDay & lhs, const JulianDay & rhs);
		friend bool operator<= (const JulianDay & lhs, const JulianDay & rhs);
		friend bool operator>= (const JulianDay & lhs, const JulianDay & rhs);
		friend bool operator!= (const JulianDay & lhs, const JulianDay & rhs);
		friend int operator- (const JulianDay & lhs, const JulianDay & rhs);

};

#endif // AUSITN_JULIANDAY_H









